package cn.rankin.apiweb.controller;

import cn.rankin.apiweb.assist.resolver.NeedUser;
import cn.rankin.apiweb.code.ApiWebCode;
import cn.rankin.apiweb.service.download.DownloadInfoService;
import cn.rankin.apiweb.service.event.EventService;
import cn.rankin.apiweb.service.product.ProductService;
import cn.rankin.common.utils.api.model.APICode;
import cn.rankin.common.utils.api.model.APIResult;
import cn.rankin.common.utils.constant.DownloadStatus;
import cn.rankin.common.utils.enums.BaseStatusEnum;
import cn.rankin.common.utils.enums.EventTypeEnum;
import cn.rankin.common.utils.util.HttpUtil;
import cn.rankin.data.api.app.vo.DeviceUserVo;
import cn.rankin.data.api.product.entity.Course;
import cn.rankin.data.api.product.entity.Lesson;
import cn.rankin.data.api.user.dto.DownloadInfoDTO;
import cn.rankin.data.api.user.dto.EventLogDTO;
import cn.rankin.data.api.user.entity.DownloadInfo;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.util.List;


/**
 * CallbackController
 * Created by huodongdong on 2017/10/27.
 */
@Slf4j
@RestController
@RequestMapping(value = "/callback")
public class CallbackController {

    @Autowired
    private EventService eventService;

    @Autowired
    private DownloadInfoService downloadInfoService;

    @Autowired
    private ProductService productService;

    @RequestMapping(value = "/event", method = RequestMethod.POST)
    public APIResult<Boolean> addEvent(@NeedUser DeviceUserVo user, @RequestBody EventLogDTO eventLog) {
        if (eventLog.getEventType() == null ) {
            return APIResult.error(ApiWebCode.PARAMETER_ERROR);
        }

        eventLog.setMerchantId(user.getMerchantId());
        eventLog.setUserId(user.getUid());

        APIResult<Boolean> result = eventService.addLog(eventLog);
        return result;
    }

    @RequestMapping(value = "/download/add", method = RequestMethod.POST)
    public APIResult<Boolean> addDownloadInfo(HttpServletRequest request, @NeedUser DeviceUserVo user ,
                                              @RequestParam("courseId") String courseId,
                                              @RequestParam("lessonId") String lessonId,
                                              @RequestParam("resTotal") int resTotal,
                                              @RequestParam("resSize") int resSize,
                                              @RequestParam("data") String data) {

        String ip = HttpUtil.getClientIp(request);
        log.info("ip={}",ip);

        String mac = HttpUtil.getMacAddrByIp(request);

        log.info("mac={}",mac);

        String systemInfo = HttpUtil.getSystemInfo(request);

        log.info("systemInfo={}",systemInfo);

        /**
         * 1.获得客户机信息
         */
        String requestUrl = request.getRequestURL().toString();//得到请求的URL地址
        String requestUri = request.getRequestURI();//得到请求的资源
        String queryString = request.getQueryString();//得到请求的URL地址中附带的参数
        String remoteAddr = request.getRemoteAddr();//得到来访者的IP地址
        String remoteHost = request.getRemoteHost();
        int remotePort = request.getRemotePort();
        String remoteUser = request.getRemoteUser();
        String method = request.getMethod();//得到请求URL地址时使用的方法
        String pathInfo = request.getPathInfo();
        String localAddr = request.getLocalAddr();//获取WEB服务器的IP地址
        String localName = request.getLocalName();//获取WEB服务器的主机名
        String device = request.getHeader("User-Agent");

        log.info("device={}",device);

        DownloadInfoDTO dto = new DownloadInfoDTO();

        if(StringUtils.isBlank(lessonId)){
            return APIResult.error(ApiWebCode.PARAMETER_ERROR);
        }else{
            Lesson lesson = productService.getLesson(lessonId, courseId).getData();
            if(null == lesson){
                log.info("not found lesson by id ,lessonId={}",lessonId);
                return APIResult.error(ApiWebCode.PARAMETER_ERROR);
            }

            dto.setLessonId(lessonId);
            dto.setLessonName(lesson.getTitle());
        }




        if(StringUtils.isBlank(courseId)){
            return APIResult.error(ApiWebCode.PARAMETER_ERROR);
        }else{
            Course course = productService.getCourse(courseId);
            if(null == course){
                log.info("not found course by id ,courseId={}",courseId);
                return APIResult.error(ApiWebCode.PARAMETER_ERROR);
            }

            dto.setCourseId(courseId);
            dto.setCourseName(course.getTitle());
        }
        if(StringUtils.isNotBlank(data)){
            dto.setData(data);
        }

        dto.setMerchantId(user.getMerchantId());
        dto.setUserId(user.getUid());
        dto.setDeviceType(device);
        dto.setDownloadStatus(DownloadStatus.PROCESS_DOWNLOAD);
        dto.setEventType(EventTypeEnum.DOWNLOAD);
        dto.setStatus(BaseStatusEnum.NORMAL);
        dto.setNeedUpdate(0);
        dto.setResTotal(resTotal);
        dto.setResSize(resSize);



        DownloadInfo downloadInfo = downloadInfoService.add(dto);
        if(downloadInfo != null){
            return APIResult.ok();
        }else{
            return APIResult.error(APICode.OPERATE_ERROR);
        }

    }

    @RequestMapping(value = "/download/list", method = RequestMethod.GET)
    public APIResult<List<DownloadInfo>> findByUserId(@NeedUser DeviceUserVo user) {
        List<DownloadInfo> downloadInfos = downloadInfoService.findByUserId(user.getUid());
        return APIResult.ok(downloadInfos);
    }

    @RequestMapping(value = "/download/find", method = RequestMethod.GET)
    public APIResult<DownloadInfo> findByLessonUserId(@NeedUser DeviceUserVo user,@RequestParam("lessonId") String lessonId) {
        DownloadInfo downloadInfo = downloadInfoService.findByLessonUserId(user.getUid(),lessonId);
        if(downloadInfo != null){
            return APIResult.ok(downloadInfo);
        }else{
            return APIResult.error(APICode.OPERATE_ERROR);
        }
    }

    @RequestMapping(value = "/download/delete", method = RequestMethod.DELETE)
    public APIResult<Boolean> deleteByLessonUserId(@NeedUser DeviceUserVo user,@RequestParam("lessonId") String lessonId) {
        DownloadInfo downloadInfo = downloadInfoService.deleteByLessonUserId(user.getUid(),lessonId);
        if(downloadInfo == null){
            APIResult.error(APICode.OPERATE_ERROR);
        }
        return APIResult.ok();
    }

    @RequestMapping(value = "/download/update", method = RequestMethod.PUT)
    public APIResult<Boolean> updateByLessonUserId(@NeedUser DeviceUserVo user,@RequestParam("lessonId") String lessonId) {
        DownloadInfo downloadInfo = downloadInfoService.updateByLessonUserId(user.getUid(),lessonId);
        if(downloadInfo == null){
            APIResult.error(APICode.OPERATE_ERROR);
        }
        return APIResult.ok();
    }

    @RequestMapping(value = "/download/update/resource/size", method = RequestMethod.PUT)
    public APIResult<Boolean> updateResourceByLessonUserId(@NeedUser DeviceUserVo user,@RequestParam("lessonId") String lessonId) {
        DownloadInfo downloadInfo = downloadInfoService.findByLessonUserId(user.getUid(),lessonId);
        if(downloadInfo == null){
            APIResult.error(APICode.OPERATE_ERROR);
        }else{
            int resSize = downloadInfo.getResSize() + 1 ;
            int resTotal = downloadInfo.getResTotal();
            if(resSize == resTotal){
                downloadInfo.setDownloadStatus(DownloadStatus.FINISH_DOWNLOAD);
            }
            downloadInfo.setResSize(resSize);
        }

        DownloadInfo update = downloadInfoService.update(downloadInfo);
        if(update == null){
            APIResult.error(APICode.OPERATE_ERROR);
        }

        return APIResult.ok();
    }

}

